import java.util.HashMap;

import javafx.animation.KeyFrame;
import javafx.animation.Timeline;
import javafx.application.Application;
import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.collections.ObservableList;
import javafx.event.Event;
import javafx.scene.Group;
import javafx.scene.LightBase;
import javafx.scene.Node;
import javafx.scene.PerspectiveCamera;
import javafx.scene.PointLight;
import javafx.scene.Scene;
import javafx.scene.SubScene;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyEvent;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.stage.Stage;
import javafx.util.Duration;

/**
 * アプリケーションクラス
 */
public class ARApplication extends Application implements ARConstant {

	/** ステージクリア */
	private final static int			STAGE_CLEAR				= 10000;

	/** フェーズ */
	private		int						phase					= 0;

	/** 時間 */
	private		int						timeCtrl				= 0;
	private		int						time					= 0;

	/** ステージ */
	private		int						stage					= 0;

	/** スコア */
	private		int						score					= 0;

	/** ハイスコア */
	private		int						scoreHigh				= 30000;

	/** モード */
	private		int						mode					= 0;

	/** ペイン */
	private		StackPane				pane					= null;

	/** シーン */
	private		Scene					scene					= null;

	/** 3Dシーン */
	private		SubScene				scene3D					= null;

	/** 2Dシーン */
	private		SubScene				scene2D					= null;

	/** カメラ */
	private		PerspectiveCamera		camera					= null;

	/** 光源 */
	private		LightBase				light					= null;

	/** パネル3D */
	private		ARPanel3D				panel3D					= null;

	/** パネル2D */
	private		Group					panel2D					= null;

	/** テキスト */
	private		Text					textKey					= null;
	private		Text					textControlStick		= null;
	private		Text					textReverse				= null;
	private		Text					textStage				= null;
	private		Text					textScore				= null;
	private		Text					textScoreHigh			= null;
	private		Text					textBy					= null;
	private		Text					textStage2				= null;
	private		Text					textGameOver			= null;

	/** キーを押している状態一覧 */
	private HashMap<KeyCode, String>	mapKeyPress				= null;

	// ポーズ中フラグ
	private		boolean					flgPause				= false;

	// タイトル
	private		ARSpriteTitleAsteroider	spriteTitleAsteroider	= null;
	private		ARSpriteTitle3D			spriteTitle3D			= null;

	/** モードセレクト */
	private		ARSpriteMode			spriteMode				= null;

	/** 自機 */
	private		ARSpriteShip			spriteShip				= null;

	/** ミニ自機 */
	private		ImageView				spriteShipMini			= null;

	/** BGM／効果音 */
	private		ARSoundMP3				mp3Title				= null;
	private		ARSoundMP3				mp3Credit				= null;
	private		ARSoundMP3[]			mp3Spaces				= null;
	private		ARSoundMP3				mp3BGM					= null;
	private		ARSoundMP3				mp3Talk1				= null;
	private		ARSoundMP3				mp3Talk2				= null;
	private		ARSoundMP3				mp3StageClear			= null;
	private		ARSoundMP3				mp3StageEnd				= null;
	private		ARSoundMP3				mp3StageStart			= null;
	private		ARSoundMP3				mp3Clash				= null;
	private		ARSoundMP3				mp3GameOver				= null;

	/** ポジション（BGM） */
	private		int						posBgm					= 0;

	/** 時間（BGM） */
	private		int						timeBgm					= 0;
	private		int						timeBgmNext				= 0;

	/**
	 * 実行
	 * <pre>
	 * 当メソッドから、はじまります。
	 * </pre>
	 * @param args 引数
	 */
	public static void main(String[] args){

		// アプリケーションを起動します
		launch(args);

	} // end main

	/**
	 * スタート
	 * <pre>
	 * 初期化を行います。
	 * </pre>
	 */
	@Override
	public void start(Stage primaryStage){

		// ObjReaderにアプリケーションを設定
		ObjReader.setApplication(this);

		// スプライト初期化
		ARSpriteMode.init();
		ARSpriteShip.init();
		ARSpriteAsteroid.init();
		ARSpriteTitleAsteroider.init();
		ARSpriteTitle3D.init();

		// ペインを生成
		pane = new StackPane();

		// シーンを生成
		// Zバッファを有効にする（最後の引数、boolean）
		scene = new Scene(pane, ARConstant.WIDTH, ARConstant.HEIGHT, true);

		// 背景を黒にする
		scene.setFill(Color.BLACK);

		// シーンを設定
		primaryStage.setScene(scene);

		// タイトル設定
		primaryStage.setTitle("ASTEROIDER");

		// ウィンドウを閉じる際に安全に終了する
		primaryStage.setOnCloseRequest(req -> {
			// JavaFXアプリケーションを終了する
			Platform.exit();
			// 他に終了していないスレッドを終了する
			System.exit(0);
		});

		// BGM・効果音を生成
		mp3Title = new ARSoundMP3(this, "sound/Asteroider.mp3", true);
		mp3Credit = new ARSoundMP3(this, "sound/waveCredit.mp3", false);
		mp3Spaces = new ARSoundMP3[4];
		mp3Spaces[0] = new ARSoundMP3(this, "sound/waveSpace0.mp3", false);
		mp3Spaces[1] = new ARSoundMP3(this, "sound/waveSpace1.mp3", false);
		mp3Spaces[2] = new ARSoundMP3(this, "sound/waveSpace2.mp3", false);
		mp3Spaces[3] = new ARSoundMP3(this, "sound/waveSpace3.mp3", false);
		mp3BGM = new ARSoundMP3(this, "sound/Asteroider2.mp3", true);
		mp3Talk1 = new ARSoundMP3(this, "sound/PasseThroughTheAsteroidBelt.mp3", false);
		mp3Talk2 = new ARSoundMP3(this, "sound/AlsoAsteroidBelt.mp3", false);
		mp3StageClear = new ARSoundMP3(this, "sound/waveStageClear.mp3", false);
		mp3StageEnd = new ARSoundMP3(this, "sound/waveStageEnd.mp3", false);
		mp3StageStart = new ARSoundMP3(this, "sound/waveStageStart.mp3", false);
		mp3Clash = new ARSoundMP3(this, "sound/waveClash.mp3", false);
		mp3GameOver = new ARSoundMP3(this, "sound/waveGameOver.mp3", false);

		// グループを生成
		panel3D = new ARPanel3D(this);

		// サブシーンを生成
		scene3D = new SubScene(panel3D, ARConstant.WIDTH, ARConstant.HEIGHT, true, null);

		// ペインに追加
		pane.getChildren().add(scene3D);

		// カメラ設定
		initCamera();

		// 光源設定
		initLight();

		// グループを生成
		panel2D = new Group();

		// スプライト一覧を生成
		scene2D = new SubScene(panel2D, ARConstant.WIDTH, ARConstant.HEIGHT);

		// スプライト層一覧に追加
		pane.getChildren().add(scene2D);

		// ミニ自機を生成
		spriteShipMini = new ImageView(new Image("2d/ShipMini.png"));

		// ミニ自機を追加
		panel2D.getChildren().add(spriteShipMini);

		// テキストを生成
		textKey = new Text(300, 500, "PUSH SPACE KEY");
		textControlStick = new Text(320, 550, "CONTROL STICK MODE");
		textReverse = new Text(320, 580, "REVERSE MODE");
		textStage = new Text(100, 20, "STAGE:" + stage);
		textScore = new Text(288, 20, "SCORE:" + score);
		textScoreHigh = new Text(512, 20, "HIGH-SCORE:" + scoreHigh);
		textBy = new Text(220, 720, "PROGRAMED  BY  KIN  /  MUSIC  BY  Aki2");
		textStage2 = new Text(336, 400, "STAGE");
		textGameOver = new Text(320, 400, "GAME OVER");

		// フォント設定
		textKey.setFont(new Font(32));
		textControlStick.setFont(new Font(20));
		textReverse.setFont(new Font(20));
		textStage.setFont(new Font(null, 20));
		textScore.setFont(new Font(null, 20));
		textScoreHigh.setFont(new Font(20));
		textBy.setFont(new Font(20));
		textStage2.setFont(new Font(32));
		textGameOver.setFont(new Font(32));

		// 色の設定
		textKey.setFill(Color.WHITE);
		textControlStick.setFill(Color.YELLOW);
		textReverse.setFill(Color.LIGHTGRAY);
		textStage.setFill(Color.WHITE);
		textScore.setFill(Color.WHITE);
		textScoreHigh.setFill(Color.WHITE);
		textBy.setFill(Color.WHITE);
		textStage2.setFill(Color.WHITE);
		textGameOver.setFill(Color.WHITE);

		// テキストを追加
		panel2D.getChildren().add(textStage);
		panel2D.getChildren().add(textScore);
		panel2D.getChildren().add(textScoreHigh);

		// 星屑分、ループ
		for(int i = 0; i < 300; i++){

			// 星屑を生成
			ARSpriteStardust spriteStardust = new ARSpriteStardust();

			// ノード追加
			this.addNode(spriteStardust);

		} // end for

		// タイトルを生成
		spriteTitleAsteroider = new ARSpriteTitleAsteroider(this);
		spriteTitle3D = new ARSpriteTitle3D(this);

		// キー押下状況一覧を生成
		mapKeyPress = new HashMap<KeyCode, String>();

		// キーを押したイベントを登録
		scene.addEventHandler(KeyEvent.KEY_PRESSED, event -> sceneKeyPressed(event));

		// キーを離したイベントを登録
		scene.addEventHandler(KeyEvent.KEY_RELEASED, event -> sceneKeyReleased(event));

		// ウインドウサイズ変更リスナー追加
		pane.widthProperty().addListener((observable, oldValue, newValue) -> windowResized());
		pane.heightProperty().addListener((observable, oldValue, newValue) -> windowResized());

		// ウィンドウ表示
		primaryStage.show();

		// 時間初期化
		timeCtrl = 0;
		time = 0;

		// タイマーをスタート
		Timeline timer = new Timeline(new KeyFrame(Duration.millis(4), event -> runCtrl()));
		timer.setCycleCount(Timeline.INDEFINITE);
		timer.play();

	} // end start

	/**
	 * タイマー実行
	 * <pre>
	 * タイマーから定期的に呼ばれます。
	 * </pre>
	 */
	synchronized public void runCtrl() {

		// 時間＋１
		timeCtrl++;

		// アプリケーション−制御の時間
		if(timeCtrl % 4 == 0){

			// アプリケーション−実行メソッド呼び出し
			run();

		// 各ノードの制御の時間
		}else if(timeCtrl % 4 == 1){

			// 3Dパネルを実行
			panel3D.run();

		// お掃除の時間
		}else if(timeCtrl % 4 == 3){

			// 3Dパネルのクリーンメソッドを実行
			panel3D.clean();

		}

	} // end runCtrl

	/**
	 * 実行
	 * <pre>
	 * タイマーから定期的に呼ばれます。
	 * </pre>
	 */
	synchronized public void run() {

		// 時間＋１
		time++;

		// フェーズ−初期化の場合、
		if(phase == PHASE_INIT){

			// ステージ初期化
			stage = 0;

			// スコア初期化
			score = 0;

			// 時間初期化
			time = 0;

			// タイトルを初期化
			spriteTitleAsteroider.initPosition();
			spriteTitle3D.initPosition();

			// フェーズ−タイトル１へ
			phase = PHASE_TITLE_1;

		// フェーズ−タイトル１の場合、
		}else if(phase == PHASE_TITLE_1){

			// ３秒
			if(time == 180){

				// タイトルを追加
				this.addNode(spriteTitleAsteroider);
				this.addNode(spriteTitle3D);

			}

			// スペースが押された場合、
			if(this.isKeyPressed(KeyCode.SPACE)){

				// ３秒未満の場合、
				if(time < 180){

					// タイトルを追加
					this.addNode(spriteTitleAsteroider);
					this.addNode(spriteTitle3D);

				}

				// タイトルをスキップ
				spriteTitleAsteroider.skipTitle();
				spriteTitle3D.skipTitle();

				// フェーズ−タイトル２へ
				phase = PHASE_TITLE_2;

			}

		// フェーズ−タイトル２の場合、
		}else if(phase == PHASE_TITLE_2){

			// タイトルから待ち

		// フェーズ−タイトル３の場合、
		}else if(phase == PHASE_TITLE_3){

			// 時間が来たら、
			if(time == 1){

				// テキストを追加
				panel2D.getChildren().add(textKey);
				panel2D.getChildren().add(textControlStick);
				panel2D.getChildren().add(textReverse);
				panel2D.getChildren().add(textBy);

				// モードセレクトを生成
				spriteMode = new ARSpriteMode(this);

				// モードセレクトを追加
				this.addNode(spriteMode);

				// BGMをスタート
				mp3Title.start();

			} // end if 時間が来たら

			// 上が押された場合、
			if(this.isKeyPressed(KeyCode.UP)){

				// モード変更
				mode = 0;

				// モード変更
				spriteMode.setMode(mode);

			// 下が押された場合、
			}else if(this.isKeyPressed(KeyCode.DOWN)){

				// モード変更
				mode = 1;

				// モード変更
				spriteMode.setMode(mode);

			// スペースが押された場合、
			}else if(this.isKeyPressed(KeyCode.SPACE)){

				// 時間初期化
				time = 0;

				// タイトルを削除
				this.removeNode(spriteTitleAsteroider);
				this.removeNode(spriteTitle3D);

				// テキストを削除
				panel2D.getChildren().remove(textKey);
				panel2D.getChildren().remove(textControlStick);
				panel2D.getChildren().remove(textReverse);
				panel2D.getChildren().remove(textBy);

				// モードセレクトを削除
				this.removeNode(spriteMode);

				// BGMをストップ
				mp3Title.stop();

				// 効果音をスタート
				mp3Credit.start();

				// 自機を生成
				spriteShip = new ARSpriteShip(this);

				// モード設定
				spriteShip.setMode(mode);

				// 自機を追加
				this.addNode(spriteShip);

				// フェーズ−ステージスタートへ
				phase = PHASE_STAGE_START;

			}

		// フェーズ−ステージスタートの場合、
		}else if(phase == PHASE_STAGE_START){

			// 時間が来たら、
			if(time == 1){

				// ステージ＋１
				stage++;

				// スコア
				textStage.setText("STAGE:" + stage);

				// テキスト−ステージを編集
				textStage2.setText("STAGE " + stage);

				// スコア
				textScore.setText("SCORE:" + score);

				// テキストを追加
				panel2D.getChildren().add(textStage2);

				// ミニ自機を移動
				spriteShipMini.setTranslateX((score % STAGE_CLEAR) / (double)STAGE_CLEAR * WIDTH);

				// 自機をスタート
				spriteShip.start();

			// ３秒、
			}else if(time == 180){

				// ステージ２以降の場合、
				if(stage >= 2){

					// 効果音をスタート
					mp3Talk2.start();

				}

			// ４秒、
			}else if(time == 240){

				// テキストを削除
				panel2D.getChildren().remove(textStage2);

				// BGMをスタート
				mp3BGM.start();

				// フェーズ−ゲーム中へ
				phase = PHASE_GAME;

			} // end if 時間が来たら

		// フェーズ−ゲーム中の場合、
		}else if(phase == PHASE_GAME){

			// スコア＋４
			score = score + 4;

			// スコア
			textScore.setText("SCORE:" + score);

			// ハイスコアを上回ったら、
			if(score > scoreHigh){

				// ハイスコア変更
				scoreHigh = score;

				// ハイスコア
				textScoreHigh.setText("HIGH-SCORE:" + scoreHigh);

			}

			// ステージクリアの場合、
			if(score % STAGE_CLEAR == 0){

				// 時間初期化
				time = 0;

				// フェーズ−ステージクリア１へ
				phase = PHASE_STAGE_CLEAR_1;

			// ステージクリア以外の場合、
			}else{

				// ミニ自機を移動
				spriteShipMini.setTranslateX((score % STAGE_CLEAR) / (double)STAGE_CLEAR * WIDTH);

			}

		// フェーズ−ステージクリア１の場合、
		}else if(phase == PHASE_STAGE_CLEAR_1){

			// 4秒
			if(time == 240){

				// 効果音をスタート
				mp3Talk1.start();

			// 8秒
			}else if(time == 480){

				// BGMをストップ
				mp3BGM.stop();

				// 効果音をスタート
				mp3StageClear.start();

			// 10秒
			}else if(time == 600){

				// 時間初期化
				time = 0;

				// 効果音をスタート
				mp3StageEnd.start();

				// ワープ
				spriteShip.warp();

				// ワープ
				ARSpriteWarp spriteWarp = new ARSpriteWarp(this, spriteShip);

				// アプリケーションにノード追加
				this.addNode(spriteWarp);

				// フェーズ−ステージクリア２へ
				phase = PHASE_STAGE_CLEAR_2;

			} // end if 時間

		// フェーズ−ステージクリア２の場合、
		}else if(phase == PHASE_STAGE_CLEAR_2){

			// 6秒
			if(time < 360){

				// 何もしない

			// 6秒
			}else{

				// 時間初期化
				time = 0;

				// 効果音をスタート
				mp3StageStart.start();

				// フェーズ−ステージスタートへ
				phase = PHASE_STAGE_START;

			} // end if 時間

		// フェーズ−クラッシュの場合、
		}else if(phase == PHASE_CLASH){

			// 時間が来たら、
			if(time > 120){

				// 効果音をスタート
				mp3GameOver.start();

				// テキストを追加
				panel2D.getChildren().add(textGameOver);

				// 時間初期化
				time = 0;

				// フェーズ−ゲームオーバーへ
				phase = PHASE_GAMEOVER;

			}

		// フェーズ−ゲームオーバーの場合、
		}else if(phase == PHASE_GAMEOVER){

			// 時間が来たら、
			if(time > 300){

				// 自機を削除
				removeNode(spriteShip);

				// 時間初期化
				time = 0;

				// テキストを削除
				panel2D.getChildren().remove(textGameOver);

				// フェーズ−初期化へ
				phase = PHASE_INIT;

			}

		} // end if フェーズ

		// フェーズ−ステージクリア以外の場合、
		if(phase != PHASE_STAGE_CLEAR_1 && phase != PHASE_STAGE_CLEAR_2){

			// 小惑星帯進行
			progress();

		}

		// フェーズ−タイトル／ゲームクリア／クラッシュ／ゲームオーバーではない場合、
		if(phase != PHASE_TITLE_1 && phase != PHASE_TITLE_2 && phase != PHASE_TITLE_3 &&
			phase != PHASE_STAGE_CLEAR_1 && phase != PHASE_STAGE_CLEAR_2 &&
			phase != PHASE_CLASH && phase != PHASE_GAMEOVER){

			// 時間（BGM）＋１
			timeBgm++;

			// 効果音挿入タイミング
			if(timeBgm > timeBgmNext){

				// 効果音を初期化
				mp3Spaces[posBgm].init();

				// 効果音挿入タイミング
				mp3Spaces[posBgm].start();

				// 時間（BGM）
				timeBgm = 0;
				// 次の時間（BGM）
				timeBgmNext = (int)(Math.random() * 300) + 300;
				// ポジション（BGM）
				posBgm = (posBgm + 1 + (int)(Math.random() * 3)) % 4;

			} // end if 効果音挿入タイミング

		} // end if フェーズ−タイトル／ゲームクリア／クラッシュ／ゲームオーバーではない場合

	} // end run

	/**
	 * 小惑星帯進行
	 * <pre>
	 * ステージにより、違った小惑星を生み出します。
	 * </pre>
	 */
	private void progress() {

		// レベル
		int lvl = stage / 3;

		// タイトル or ステージ１系の場合、
		if(stage == 0 || stage % 3 == 1){

			// 小惑星を出現させる場合、
			if(Math.random() < 0.15 + lvl * 0.05){

				// 小惑星を生成
				ARSpriteAsteroid spriteAsteroid = new ARSpriteAsteroid(this, stage);

				// ノード追加
				this.addNode(spriteAsteroid);

			}

		// ステージ２系の場合、
		}else if(stage % 3 == 2){

			// 小惑星を出現させる場合、
			if(Math.random() < 0.40 + lvl * 0.20){

				// 小惑星を生成
				ARSpriteAsteroid spriteAsteroid = new ARSpriteAsteroid(this, stage);

				// ノード追加
				this.addNode(spriteAsteroid);

			}

		// ステージ３系の場合、
		}else if(stage % 3 == 0){

			// 小惑星を出現させる場合、
			if(Math.random() < 0.06 + lvl * 0.03){

				// 小惑星を生成
				ARSpriteAsteroid spriteAsteroid = new ARSpriteAsteroid(this, stage);

				// ノード追加
				this.addNode(spriteAsteroid);

			}

		} // end if ステージ

	} // end progress

	/**
	 * フェーズ設定
	 */
	public void setPhase(int phase){

		// 時間初期化
		time = 0;

		// フェーズ変更
		this.phase = phase;

	} // end setPhase

	/**
	 * ポーズ中か取得
	 * @return boolean true−ポーズ中／false−ポーズ中ではない
	 */
	public boolean isPause(){
		return flgPause;
	}

	/**
	 * ノード追加
	 * <pre>
	 * ノードを追加します。
	 * </pre>
	 * @param sprite ノード
	 */
	public void addNode(Node node){

		// 3Dパネルクラスを呼び出し
		panel3D.addNode(node);

	}

	/**
	 * ノード削除
	 * <pre>
	 * ノードを削除します。
	 * 当メソッドでは、ダミーノードに置き換え、runメソッドの中で削除します。
	 * nullに置き換えができない
     * かつ Listクラスにremoveメソッドを多用することで配列コピーを多用されることを避けるため
	 * （重くなることをできるだけ避ける）
	 * </pre>
	 * @param sprite ノード
	 */
	public void removeNode(Node node){

		// 3Dパネルクラスを呼び出し
		panel3D.removeNode(node);

	}

	/**
	 * メディアビュー追加
	 * @param sprite スプライト
	 */
	public void addMediaView(ARSoundMP3 mp3){

		// スプライト一覧を取得
		ObservableList<Node> nodeLst = pane.getChildren();

		// メディアビューを追加
		nodeLst.add(mp3);

	} // end addMediaView

	/**
	 * メディアビュー削除
	 * <pre>
	 * スプライト層から探索して、スプライトを削除します。
	 * 当メソッドでは、ダミースプライトに置き換え、runメソッドの中で削除します。
	 * nullに置き換えができない
     * かつ Listクラスにremoveメソッドを多用することで配列コピーを多用されることを避けるため
	 * （重くなることをできるだけ避ける）
	 * </pre>
	 * @param sprite スプライト
	 */
	public void removeMediaView(ARSoundMP3 mp3){

		// スプライト一覧を取得
		ObservableList<Node> nodeLst = pane.getChildren();

		// インデックスを求める
		int index = nodeLst.indexOf(mp3);

		// インデックスが取得できた場合、
		if(index != -1){

			// 削除
			nodeLst.remove(mp3);

		}

	} // end removeMediaView

	/**
	 * 自機取得
	 */
	public ARSpriteShip getARSpriteShip(){
		return spriteShip;
	}

	/**
	 * クラッシュ
	 */
	public void clash(){

		// 自機をクラッシュさせる
		spriteShip.clash();

		// BGMをストップ
		mp3BGM.stop();

		// 効果音をスタート
		mp3Clash.start();

		// 時間初期化
		time = 0;

		// フェーズ−クラッシュへ
		phase = PHASE_CLASH;

	} // end clash

	/**
	 * カメラ設定
	 */
	private void initCamera(){

		// カメラ設定
		camera  = new PerspectiveCamera(true);

		// カメラの空間を設定
		camera.setFarClip(300);

		// カメラを設定
		scene3D.setCamera(camera);

	} // end initCamera

	/**
	 * 光源設定
	 */
	private void initLight(){

		// 光源設定
		light = new PointLight();

		// 光源位置
		light.setTranslateX(0);
		light.setTranslateY(0);
		light.setTranslateZ(0);

		// 光源を設定
		panel3D.getChildren().add(light);

	} // end initLight

	/**
	 * キー押下中かをどうかをチェック
	 * @param code キーコード
	 * @return boolean true：押している ／ false：押していない
	 */
	public boolean isKeyPressed(KeyCode code){

		return mapKeyPress.containsKey(code);

	} // end isKeyPressed

	/**
	 * キーを押したイベント
	 * <pre>
	 * ラムダ式記述を行って、インターフェース作成を省略しています。
	 * </pre>
	 */
	public void sceneKeyPressed(Event e){

		KeyEvent ke = (KeyEvent)e;
		KeyCode code = ke.getCode();
		mapKeyPress.put(code, "");

	} // end sceneKeyPressed

	/**
	 * キーを離したイベント
	 * <pre>
	 * ラムダ式記述を行って、インターフェース作成を省略しています。
	 * </pre>
	 */
	public void sceneKeyReleased(Event e){

		KeyEvent ke = (KeyEvent)e;
		KeyCode code = ke.getCode();
		mapKeyPress.remove(code);

	} // end sceneKeyReleased

	/**
	 * ウインドウサイズ変更イベント
	 */
	public void windowResized(){

		// ペイン（ウインドウ）のサイズを取得
		double widthPane = ARApplication.this.pane.getWidth();
		double heightPane = ARApplication.this.pane.getHeight();

		// 比率を求める
		double ratioWidth = (float)widthPane / ARConstant.WIDTH;
		double ratioHeight = (float)heightPane / ARConstant.HEIGHT;

		// 比率の小さい方を求める
		double ratio = (ratioWidth < ratioHeight)?ratioWidth: ratioHeight;

		// 各シーンを拡大縮小する
		scene3D.setScaleX(ratio);
		scene3D.setScaleY(ratio);
		scene2D.setScaleX(ratio);
		scene2D.setScaleY(ratio);

		// ペイン上のシーンを再配置する
		scene3D.setLayoutX((widthPane - ARConstant.WIDTH) / 2);
		scene3D.setLayoutY((heightPane - ARConstant.HEIGHT) / 2);
		scene2D.setLayoutX((widthPane - ARConstant.WIDTH) / 2);
		scene2D.setLayoutY((heightPane - ARConstant.HEIGHT) / 2);

	} // end windowResized

}
