import javafx.scene.Group;

/**
 * ダミーノード
 * <pre>
 * 削除されたノードは、一旦、ダミーノードに置き換えます。
 * その後、別タイミングにて、実際の削除を実施します。
 * JavaFXでは、childをnullに置き換えることができないため、苦肉の策で、このような形になりました。
 * nullに置き換えずに、removeを実施すると、要素数が動的に減ることになり、
 * forループにおいて、次のループ時に飛び越えが発生し、その要素の処理が行われない状況が発生します。
 * その状況を回避するための苦肉の策です。
 * <pre>
 */
public class ARNodeDummy extends Group {

	/**
	 * コンストラクタ
	 * <pre>
	 * ノードの連携が必要ない場合は、当コンストラクタを使用ください。
	 * </pre>
	 * @param panel パネル
	 */
	public ARNodeDummy(ARApplication applicatin) {

		// 表示もしないようにする
		super.setVisible(false);

	}

	/**
	 * 実行
	 * <pre>
	 * 当オブジェクトのふるまいを決定します。
	 * 当ダミーノードクラスでは、なにも行いません（スーパークラスも呼び出さない）
	 * <pre>
	 */
	public void run(){

	} // end run

	/**
	 * 幅取得
	 * <pre>
	 * 当ダミーノードクラスでは、0を返却します。
	 * スーパークラスを呼び出すと、NullPointerExceptionが発生するため、当クラスでダミーで返却します。
	 * <pre>
	 * @return double 幅 0を返却
	 */
	public double getWidth(){
		return 0;
	}

	/**
	 * 高さ取得
	 * <pre>
	 * 当ダミーノードクラスでは、0を返却します。
	 * スーパークラスを呼び出すと、NullPointerExceptionが発生するため、当クラスでダミーで返却します。
	 * <pre>
	 * @return double 高さ 0を返却
	 */
	public double getHeight(){
		return 0;
	}

}
