import javafx.geometry.Point3D;

import javafx.geometry.BoundingBox;
import javafx.geometry.Bounds;

/**
 * アステロイド
 */
public class ARSpriteAsteroid extends ARSpriteT00 {

	/** 回転速度 */
	private		double			s			= Math.random() * 4 + 2;

	/** 速度X */
	private		double			mx			= Math.random() * 0.1 - 0.05;

	/** 速度Y */
	private		double			my			= Math.random() * 0.1 - 0.05;

	/** 速度Z */
	private		double			mz			= -Math.random() * 0.8 - 0.4;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(ARSpriteAsteroid.class, "3d/Asteroid2.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param applicaton ARApplication
	 * @param stage ステージ
	 */
	public ARSpriteAsteroid(ARApplication applicaton, int stage){

		// スーパークラスを呼び出す
		super(applicaton);

		try{

			// パターン設定
			int pattern = 0;
			if(stage == 0){
				pattern = 1;
			}else if(stage % 3 == 0){
				pattern = 3;
			}else{
				pattern = stage % 3;
			}

			// レベル
			int lvl = stage / 3;

			// サイズ設定
			double size = 0;

			// パターン別設定
			switch(pattern){

				// ステージ１系
				case 1:

					// サイズ
					size = 1.0;

					// ブレイク
					break;

				// ステージ２系
				case 2:

					// サイズ
					size = 0.6;

					// 回転速度
					s = Math.random() * 5 + 5;

					// 速度
					mx = Math.random() * 0.16 - 0.08;
					my = Math.random() * 0.16 - 0.08;
					mz = mz * (1 + lvl * 0.5);

					// ブレイク
					break;

				// ステージ３系
				case 3:

					// サイズ
					size = 2.0;

					// 回転速度
					s = Math.random() * 2 + 1;

					// 速度
					mx = Math.random() * 0.06 - 0.03;
					my = Math.random() * 0.06 - 0.03;
					mz = -Math.random() * 0.4 - 0.2;

					// ブレイク
					break;

				// その他の場合、
				default:

					// エラーをスロー
					throw new IllegalStateException("pattern not found.");

			} // end switch パターン別設定

			// X設定
			super.setTranslateX(Math.random() * 20 - 10);

			// Y設定
			super.setTranslateY(Math.random() * 20 - 10);

			// Z設定
			super.setTranslateZ(300);

			// オブジェクト
			ObjReader.create(this);

			// 回転軸
			Point3D p = new Point3D(Math.random() - 0.5, Math.random() - 0.5, Math.random() - 0.5);

			// 回転軸を設定
			super.setRotationAxis(p);

			// 拡大／縮小する
			super.setScaleX(size);
			super.setScaleY(size);
			super.setScaleZ(size);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end ARSpriteAsteroid

	/**
	 * 実行
	 */
	public void run(){

		// 座標取得
		double x = super.getTranslateX();
		double y = super.getTranslateY();
		double z = super.getTranslateZ();

		// 座標加算
		x = x + mx;
		y = y + my;
		z = z + mz;

		// 画面の外にでた場合、
		if(z < 10){
			// 削除する
			super.remove();
		}

		// 移動
		super.setTranslateX(x);
		super.setTranslateY(y);
		super.setTranslateZ(z);

		// 回転
		super.setRotate(getRotate() + this.s);

		//
		// 当たり判定
		//

		// Ｚ方向として当たり判定が必要な場合、
		if(z < 12){

			// 境界を取得
			Bounds boundsThis = this.getBoundsInParent();

			// 当たり判定が厳しいので、少し緩めにする
			// Boundsは、protectedで生成できないので、BoundingBoxで生成する
			boundsThis = new BoundingBox(
				boundsThis.getMinX() + boundsThis.getWidth() * 0.25,
				boundsThis.getMinY() + boundsThis.getHeight() * 0.25,
				boundsThis.getMinZ(),
				boundsThis.getWidth() * 0.5,
				boundsThis.getHeight() * 0.5,
				boundsThis.getDepth()
			);

			// 自機を取得
			ARSpriteShip spriteShip = application.getARSpriteShip();

			// 自機が取得できない場合、
			if(spriteShip == null){
				// リターン 
				return;
			}

			// 境界を取得
			Bounds boundsShip = spriteShip.getBoundsInParent();

			// 当たり判定が厳しいので、少し緩めにする
			boundsShip = new BoundingBox(
				boundsShip.getMinX() + boundsShip.getWidth() * 0.25,
				boundsShip.getMinY() + boundsShip.getHeight() * 0.25,
				boundsShip.getMinZ(),
				boundsShip.getWidth() * 0.5,
				boundsShip.getHeight() * 0.5,
				boundsShip.getDepth()
			);

			// 当たっている場合、
			if(boundsThis.intersects(boundsShip)){

				// 無敵状態 ／ クラッシュ ではない場合、
				if(!spriteShip.isMuteki() && !spriteShip.isClash()){

					// 自機クラッシュ
					application.clash();

				}

			} // end if 当たっている場合

		} // end if Ｚ方向として当たり判定が必要な場合

	} // end run

}
