import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.Sphere;

/**
 * 爆発
 */
public class ARSpriteBomb extends ARSprite {

	/** 球の色 */
	private static final Color[]		COLORS			= new Color[]{
		Color.color(0.6, 0.6, 0.0, 0.2),
		Color.color(0.6, 0.6, 0.0, 0.4),
		Color.color(0.8, 0.8, 0.0, 0.6),
		Color.color(0.8, 0.8, 0.0, 0.8),
		Color.color(0.9, 0.9, 0.0, 1.0),
		Color.color(0.8, 0.8, 0.0, 0.8),
		Color.color(0.8, 0.8, 0.0, 0.6),
		Color.color(0.6, 0.6, 0.0, 0.4),
		Color.color(0.6, 0.6, 0.0, 0.2)
	};

	/** アプリケーション */
	public		ARApplication			application		= null;

	/** 爆発させるスプライト */
	public		ARSprite				sprite			= null;

	/** 球 */
	private		Sphere					sphere			= null;

	/** マテリアル */
	private		PhongMaterial			material		= null;

	/** 時間 */
	private		int						time			= 0;

	/** 半径増分 */
	private		double					aRadius			= 0;

	/** 色位置 */
	private		int						posColors		= 0;

	/**
	 * コンストラクタ
	 * @param application ARApplication
	 * @param group Group 爆発させるスプライト
	 */
	public ARSpriteBomb(ARApplication application, ARSprite sprite) {

		super(application);

		// アプリケーションを保持
		this.application = application;

		// 爆発させるスプライトを保持
		this.sprite = sprite;

		// 場所を設定
		this.setTranslateX(sprite.getTranslateX());
		this.setTranslateY(sprite.getTranslateY());
		this.setTranslateZ(sprite.getTranslateZ() - 1);

		// 半径増分を計算
		aRadius = 0.02;

		// 球を生成
		sphere = new Sphere(0.0);

		// マテリアルを生成
		material = new PhongMaterial(COLORS[0]);

		// マテリアルを設定
		sphere.setMaterial(material);

		// 球を追加
		this.getChildren().add(sphere);

	} // end ARSpriteBomb

	/**
	 * 実行
	 */
	public void run(){

		// 時間＋１
		time++;

		// 0.5秒
		if(time < 30){

			// 半径を設定
			sphere.setRadius(sphere.getRadius() + aRadius);

		// 0.5秒超
		}else{

			// 0.2秒単位の場合、
			if(time % 12 == 0){

				// 色位置＋１
				posColors++;

				// 色が無くなった場合、
				if(posColors >= COLORS.length){

					// アプリケーションからノード削除
					application.removeNode(this);

					// リターン
					return;

				}

				// 色を変更
				material.setDiffuseColor(COLORS[posColors]);

				// 色位置が４になった場合、
				if(posColors == 4){

					// 爆発させるスプライトを削除
					application.removeNode(sprite);

				}

			} // end if // 0.2秒単位の場合

		} // end if 0.5秒超

	} // end run

}
