import javafx.scene.transform.Rotate;

/**
 * モードセレクト
 */
public class ARSpriteMode extends ARSprite {

	/** モード */
	private		int				mode			= 0;

	/** 傾きY */
	private		Rotate			rotateY			= null;
	private		int				angleY			= 0;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(ARSpriteMode.class, "3d/ship.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 */
	public ARSpriteMode(ARApplication applicaton){

		// スーパークラスを呼び出す
		super(applicaton);

		try{

			// 座標設定
			super.setTranslateX(-0.7);
			super.setTranslateY(1.0);
			super.setTranslateZ(10);

			// オブジェクト
			ObjReader.create(this);

			// 縮小する
			super.setScaleX(0.02);
			super.setScaleY(0.02);
			super.setScaleZ(0.02);

			// 回転させる
			this.getTransforms().add(new Rotate(180, Rotate.Y_AXIS));

			// Rotateを生成
			rotateY = new Rotate(angleY, Rotate.X_AXIS);

			// 回転用に登録
			this.getTransforms().add(rotateY);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end ARSpriteMode

	/**
	 * 実行
	 */
	public void run(){

		// 角度を計算
		angleY = angleY + ((mode == 0)?-4: 4);

		angleY = (angleY > 360)?angleY - 360: angleY;
		angleY = (angleY < 0)?angleY + 360: angleY;

		// 角度を設定
		rotateY.setAngle(angleY);

	} // end run

	/**
	 * モード設定
	 */
	public void setMode(int mode){

		// モードを保持
		this.mode = mode;

		// 位置設定
		super.setTranslateY(1.0 + mode * 0.2);

	} // end run

}
