import javafx.scene.input.KeyCode;
import javafx.scene.transform.Rotate;

/**
 * 自機
 */
public class ARSpriteShip extends ARSprite {

	/** フェーズ−スタート */
	private final int			PHASE_START		= 0;

	/** フェーズ−ゲーム中 */
	private final int			PHASE_GAME		= 1;

	/** フェーズ−クラッシュ */
	private final int			PHASE_CLASH		= 2;

	/** フェーズ−ワープ */
	private final int			PHASE_WARP		= 3;

	/** フェーズ */
	private		int				phase			= PHASE_START;

	/** 時間 */
	private		int				time			= 0;

	/** 速度X */
	private		double			mx				= 0;

	/** 速度Y */
	private		double			my				= 0;

	/** 傾きX */
	private		Rotate			rotateX			= null;
	private		int				angleX			= 0;

	/** 傾きY */
	private		Rotate			rotateY			= null;
	private		int				angleY			= 0;

	/** 上移動 */
	private		KeyCode			moveUp			= null;

	/** 下移動 */
	private		KeyCode			moveDown		= null;

	/** 無敵状態 */
	private		boolean			flgMuteki		= false;

	/** クラッシュ状態 */
	private		boolean			flgClash		= false;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(ARSpriteShip.class, "3d/ship.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 */
	public ARSpriteShip(ARApplication applicaton){

		// スーパークラスを呼び出す
		super(applicaton);

		try{

			// Z設定
			super.setTranslateZ(10);

			// オブジェクト
			ObjReader.create(this);

			// 縮小する
			super.setScaleX(0.1);
			super.setScaleY(0.1);
			super.setScaleZ(0.1);

			// 回転させる
			this.getTransforms().add(new Rotate(180, Rotate.Y_AXIS));

			// Rotateを生成
			rotateX = new Rotate(angleX, Rotate.Z_AXIS);
			rotateY = new Rotate(angleY, Rotate.X_AXIS);

			// 回転用に登録
			this.getTransforms().add(rotateX);
			this.getTransforms().add(rotateY);

			// 無敵状態
			flgMuteki = true;

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end ARSpriteShip

	/**
	 * 実行
	 */
	public void run(){

		// 時間＋１
		time++;

		// フェーズ−スタート
		if(phase == PHASE_START){

			// 座標
			double l = 0;

			// プラスマイナス計算
			int a = (this.getLayoutX() > 0)?-1: 1;

			// ２秒
			if(time < 120){

				// 座標
				l = 0.8;

			// ４秒
			}else if(time < 240){

				// 座標を計算
				l = 0.8 - (time - 120) / 120.0 * 0.8;

			// ４秒
			}else{

				// 座標
				l = 0;

				// 時間を初期化
				time = 0;

				// フェーズ−ゲーム中へ
				phase = PHASE_GAME;

			}

			// 座標を計算
			double x = l * Math.cos(time * 0.6) * a;
			double y = l * Math.sin(time * 0.6) * a;
			double z = 10.0 - l * 8.0;

			// 自機の場所を設定
			this.setTranslateX(x);
			this.setTranslateY(y);
			this.setTranslateZ(z);

		// フェーズ−ゲーム中
		}else if(phase == PHASE_GAME){

			// 座標を取得
			double x = this.getTranslateX();
			double y = this.getTranslateY();

			// 上移動の場合、
			if(application.isKeyPressed(moveUp)){

				// 速度を計算
				my = my + ((my > -0.08)?-0.002: 0);

				// 傾きを計算
				angleY--;

			} // end if 上移動の場合

			// 下移動の場合、
			if(application.isKeyPressed(moveDown)){

				// 速度を計算
				my = my + ((my < 0.08)?0.002: 0);

				// 傾きを計算
				angleY++;

			} // end if 下移動の場合

			// 左が押された場合、
			if(application.isKeyPressed(KeyCode.LEFT)){

				// 速度を計算
				mx = mx + ((mx > -0.08)?-0.002: 0);

				// 傾きを計算
				angleX++;

			} // end if 左下が押された場合

			// 右が押された場合、
			if(application.isKeyPressed(KeyCode.RIGHT)){

				// 速度を計算
				mx = mx + ((mx < 0.08)?0.002: 0);

				// 傾きを計算
				angleX--;

			} // end if 右が押された場合

			// 座標計算
			x = x + mx;
			y = y + my;

			// 領域外の場合、
			if(y < -2.0){

				// 戻す
				y = -2.0;
				my = 0.0;
				angleY = 0;

			}

			// 領域外の場合、
			if(y > 2.0){

				// 戻す
				y = 2.0;
				my = 0.0;
				angleY = 0;

			}

			// 領域外の場合、
			if(x < -2.0){

				// 戻す
				x = -2.0;
				mx = 0.0;
				angleX = 0;

			}

			// 領域外の場合、
			if(x > 2.0){

				// 戻す
				x = 2.0;
				mx = 0.0;
				angleX = 0;

			}

			// 座標を設定
			this.setTranslateX(x);
			this.setTranslateY(y);

			// 角度を設定
			rotateX.setAngle(angleX);
			rotateY.setAngle(angleY);

			// ３秒
			if(time < 180){

				// 点滅
				if(time % 25 < 10){
					this.setVisible(false);
				}else{
					this.setVisible(true);

				}

			// ３秒
			}else if(time == 180){

				// 表示
				this.setVisible(true);

				// 無敵状態解除
				flgMuteki = false;

			} // end if

		// フェーズ−クラッシュ
		}else if(phase == PHASE_CLASH){

			// なにもしない

		// フェーズ−ワープ
		}else if(phase == PHASE_WARP){

			// １．５秒
			if(time < 90){

				// なにもしない

			// ３．０秒
			}else if(time < 180){

				// 時間を計算
				int time1 = time - 90;

				// Z計算
				this.setTranslateZ((time1 % 10) * time1 / 30 + 10);

			// ３．０秒
			}else if(time == 180){

				// Z座標を一旦戻す
				this.setTranslateZ(10);

			// ４．５秒
			}else if(time < 270){

				// Z計算
				this.setTranslateZ(this.getTranslateZ() + 20);

			}

		} // end if フェーズ

	} // end run

	/**
	 * 無敵状態取得
	 */
	public boolean isMuteki(){
		return flgMuteki;
	}

	/**
	 * クラッシュ状態取得
	 */
	public boolean isClash(){
		return flgClash;
	}

	/**
	 * クラッシュ
	 */
	public void clash(){

		// クラッシュ状態
		flgClash = true;

		// 爆発を生成
		ARSpriteBomb spriteBomb = new ARSpriteBomb(application, this);

		// アプリケーションに爆発を追加
		application.addNode(spriteBomb);

		// フェーズ−クラッシュへ
		phase = PHASE_CLASH;

	} // end clash

	/**
	 * スタート
	 */
	public void start(){

		// 時間を初期化
		time = 0;

		// 各情報を初期化
		mx = 0;
		my = 0;
		angleY = 0;
		angleX = 0;
		rotateX.setAngle(angleX);
		rotateY.setAngle(angleY);
		flgMuteki = true;

		// フェーズ−スタートへ
		phase = PHASE_START;

	} // end start

	/**
	 * ワープ
	 */
	public void warp(){

		// 時間を初期化
		time = 0;

		// フェーズ−ワープへ
		phase = PHASE_WARP;

	} // end warp

	/**
	 * モード設定
	 */
	public void setMode(int mode){

		// モード設定
		if(mode == 0){
			moveUp = KeyCode.DOWN;
			moveDown = KeyCode.UP;
		}else{
			moveUp = KeyCode.UP;
			moveDown = KeyCode.DOWN;
		}

	} // end run

}
