import javafx.geometry.Point3D;

/**
 * タイトル−3D
 */
public class ARSpriteTitle3D extends ARSprite implements ARConstant {

	/** フェーズ */
	private		int				phase		= 0;

	/** 時間 */
	private		int				time		= 0;

	/** 回転速度 */
	private		double			s			= 6.0;

	/** 速度X */
	private		double			mx			= 0.32;

	/** 速度Z */
	private		double			mz			= -1.6;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(ARSpriteTitle3D.class, "3d/3D.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application ARApplication
	 */
	public ARSpriteTitle3D(ARApplication application){

		// スーパークラスを呼び出す
		super(application);

		try{

			// オブジェクト
			ObjReader.create(this);

			// 拡大する
			super.setScaleX(1.5);
			super.setScaleY(1.5);
			super.setScaleZ(1.5);

			// Y設定
			super.setTranslateY(0.2);

			// 回転軸
			Point3D p = new Point3D(0.0, 1.0, 0.0);

			// 回転
			super.setRotate(45);

			// 回転軸を設定
			super.setRotationAxis(p);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end ARSpriteAsteroid

	/**
	 * 実行
	 */
	public void run(){

		// フェーズ０
		if(phase == 0){

			// 時間初期化
			time = 0;

			// フェーズ１へ
			phase = 1;

		// フェーズ１
		}else if(phase == 1){

			// 座標取得
			double x = super.getTranslateX();
			double z = super.getTranslateZ();

			// 定位置に来るまで、
			if(z > 14){

				// 座標加算
				x = x + mx;
				z = z + mz;

				// 画面の外にでた場合、
				if(z < 0){
					// 非表示にする
					super.setVisible(false);;
				}

				// 移動
				super.setTranslateX(x);
				super.setTranslateZ(z);

			// 定位置に来たら、
			}else{
				// 時間初期化
				time = 0;
				// フェーズ２へ
				phase = 2;
			}

		// フェーズ２
		}else if(phase == 2){

			// 時間＋１
			time++;

			// 時間が経ったら、
			if(time > 60){

				// 時間初期化
				time = 0;
				// フェーズ３へ
				phase = 3;

			}

		// フェーズ３
		}else if(phase == 3){

			// 回転
			super.setRotate(getRotate() + this.s);

		}

	} // end run

	/**
	 * 初期化
	 */
	public void initPosition(){

		// フェーズ０
		phase = 0;

		// X設定
		super.setTranslateX(-55.5);

		// Z設定
		super.setTranslateZ(300);

		// 回転
		super.setRotate(0);

	} // end initPosition

	/**
	 * タイトルをスキップ
	 */
	public void skipTitle(){

		// Y設定
		super.setTranslateX(1.78);
		// Z設定
		super.setTranslateZ(13.6);

		// 時間初期化
		time = 0;

		// フェーズ２へ
		phase = 2;

	} // end skipTitle

}
