import javafx.geometry.Point3D;

/**
 * タイトル−アステロイダー
 */
public class ARSpriteTitleAsteroider extends ARSprite implements ARConstant {

	/** フェーズ */
	private		int				phase		= 0;

	/** 時間 */
	private		int				time		= 0;

	/** 回転速度 */
	private		double			s			= 2.0;

	/** 速度Y */
	private		double			my			= 0.32;

	/** 速度Z */
	private		double			mz			= -1.6;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(ARSpriteTitleAsteroider.class, "3d/ASTEROIDER.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application ARApplication
	 */
	public ARSpriteTitleAsteroider(ARApplication application){

		// スーパークラスを呼び出す
		super(application);

		try{

			// オブジェクト
			ObjReader.create(this);

			// 回転軸
			Point3D p = new Point3D(0.0, -0.1, 0.0);

			// 回転軸を設定
			super.setRotationAxis(p);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end ARSpriteAsteroid

	/**
	 * 実行
	 */
	public void run(){

		// フェーズ０
		if(phase == 0){

			// 時間初期化
			time = 0;

			// フェーズ１へ
			phase = 1;

		// フェーズ１
		}else if(phase == 1){

			// 座標取得
			double y = super.getTranslateY();
			double z = super.getTranslateZ();

			// 定位置に来るまで、
			if(z > 14){

				// 座標加算
				y = y + my;
				z = z + mz;

				// 画面の外にでた場合、
				if(z < 0){
					// 非表示にする
					super.setVisible(false);;
				}

				// 移動
				super.setTranslateY(y);
				super.setTranslateZ(z);

			// 定位置に来たら、
			}else{
				// 時間初期化
				time = 0;
				// フェーズ２へ
				phase = 2;
			}

		// フェーズ２
		}else if(phase == 2){

			// 時間＋１
			time++;

			// 時間が経ったら、
			if(time > 60){

				// 時間初期化
				time = 0;
				// フェーズ３へ
				phase = 3;

				// フェーズ設定
				super.application.setPhase(PHASE_TITLE_3);

			}

		// フェーズ３
		}else if(phase == 3){

			// 回転
			super.setRotate(getRotate() + this.s);

		}

	} // end run

	/**
	 * 初期化
	 */
	public void initPosition(){

		// フェーズ０
		phase = 0;

		// Y設定
		super.setTranslateY(-58.0);

		// Z設定
		super.setTranslateZ(300);

		// 回転
		super.setRotate(0);

	} // end initPosition

	/**
	 * タイトルをスキップ
	 */
	public void skipTitle(){

		// Y設定
		super.setTranslateY(-0.72);
		// Z設定
		super.setTranslateZ(13.6);

		// 時間初期化
		time = 0;

		// フェーズ２へ
		phase = 2;

	} // end skipTitle

}
